//
//   File : optw_lag.cpp
//   Creation date : Wed Cct 16 10:45:54 CEST 2002 by Juanjo lvarez (juanjux@yahoo.es)
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2002 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//


#include "optw_lag.h"
#include "qtooltip.h"

#include <qlayout.h>

#include "kvi_options.h"
#include "kvi_locale.h"

//#warning "Info tips"


KviLagOptionsWidget::KviLagOptionsWidget(QWidget * parent)
: KviOptionsWidget(parent,"lag_options_widget")
{
	createLayout(5,1);

	KviBoolSelector * pUse = addBoolSelector(0,0,0,0, __tr("Use Lagmetering Engine"), KviOption_boolUseLagMeterEngine);
    KviUIntSelector * pInterval = addUIntSelector(0,1,0,1, __tr("Lag Checking Interval (in seconds)"), KviOption_uintLagCheckingInterval, 2, 10000, 15, KVI_OPTION_BOOL(KviOption_boolUseLagMeterEngine));
    connect(pUse,SIGNAL(toggled(bool)),pInterval,SLOT(setEnabled(bool)));

    KviUIntSelector * pAlarm = addUIntSelector(0,2,0,2, __tr("Trigger OnLagAlarmTimeUp event when lag is more than... (msecs)"), KviOption_uintLagAlarmTime, 1000, 1000000, 30000, KVI_OPTION_BOOL(KviOption_boolUseLagMeterEngine));
    connect(pUse,SIGNAL(toggled(bool)),pAlarm,SLOT(setEnabled(bool)));

    KviBoolSelector * pShow = addBoolSelector(0,3,0,3, __tr("Show lag in IRC context display"), KviOption_boolShowLagOnContextDisplay, KVI_OPTION_BOOL(KviOption_boolUseLagMeterEngine));
    connect(pUse,SIGNAL(toggled(bool)),pShow,SLOT(setEnabled(bool)));

	addRowSpacer(0,4,0,4);

#ifdef COMPILE_INFO_TIPS
    QToolTip::add(pUse,__c2q(__tr("<center>If you enable this, the lag metering engine, which checks at regular intervals how much lag (latency) the server has, will be enabled and you could configure more options of the engine.</center>")));
    QToolTip::add(pInterval,__c2q(__tr("<center>This specifies the time in seconds that the lag meter will wait between two lag checks. Please note that setting this option time too low could actually cause lag.</center>")));
    QToolTip::add(pAlarm,__c2q(__tr("<center>This specifies the time in miliseconds that will trigger the OnLagAlarmTimeUp event that may be used in scripts for taking actions when the lag is too high (example: reconnecting to the server). Once this level is reached, the next time the lag go back to a lower level than this time the event OnLagAlarmTimeDown will be triggered.</center>")));

    QToolTip::add(pShow,__c2q(__tr("<center>This will made the IRC context display applet to show the current lag after the user's nickname (in seconds)</center>")));
#endif
}

KviLagOptionsWidget::~KviLagOptionsWidget()
{
}


#include "m_optw_lag.moc"
