//
//   File : libkvisetup.cpp
//   Creation date : Sat Oct  6 02:08:24 2001 GMT by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2001 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "kvi_module.h"

#include "setupwizard.h"

#include <qstring.h>

#include "kvi_options.h"

// this will be choosen during the setup process
KviStr g_szChoosenIncomingDirectory;


static void unregisterLocalMetaObjects()
{
	KviModule::unregisterMetaObject("KviSetupWizard");
	KviModule::unregisterMetaObject("KviSetupPage");
	KviModule::unregisterMetaObject("DirectoryView");
}

// this is called by KviApp when no local kvirc directory has been found
// we have nothing to rely on yet... (settings have not been loaded yet)
KVIMODULEEXPORTFUNC bool setup_begin()
{
	KviSetupWizard * w = new KviSetupWizard();
	int retcode = w->exec();
	delete w;
	unregisterLocalMetaObjects();
	return (retcode == QDialog::Accepted);
}


// this is called just after the first startup
// all the subsystems are up and running (we even have a nice console
// to write on if needed).
// all the options are loaded.
KVIMODULEEXPORTFUNC void setup_finish()
{
	KVI_OPTION_STRING(KviOption_stringIncomingPath) = g_szChoosenIncomingDirectory;
}

static bool setup_module_init(KviModule *m)
{
	return true;
}

static bool setup_module_cleanup(KviModule *m)
{
	unregisterLocalMetaObjects();
	return true;
}

static bool setup_module_can_unload(KviModule *m)
{
	return true; // can be always unloaded...this module does basically nothing
}


KVIMODULEEXPORTDATA KviModuleInfo kvirc_module_info =
{
    "Setup wizard",
	"1.0.0",
	"Szymon Stefanek <stefanek@tin.it>" ,
	"First-time-run setup wizard",
    setup_module_init ,
    setup_module_can_unload,
    0,
	setup_module_cleanup
};
